from typing import Optional, List, Iterable, Dict

from airflow.providers.cncf.kubernetes.operators.kubernetes_pod import KubernetesPodOperator
from kubernetes.client import models as k8s


default_resources = k8s.V1ResourceRequirements(
    requests=dict(cpu="100Mi", memory="1Gi"),
    limits=dict(cpu="1", memory="2Gi"),
)


class BaseSparkKubernetesPodOperator(KubernetesPodOperator):
    template_fields: Iterable[str] = (
        'image',
        'cmds',
        'arguments',
        'env_vars',
        'labels',
    )

    def __init__(
            self,
            namespace: str,
            image: str,
            name: Optional[str] = None,
            cmds: Optional[List[str]] = None,
            arguments: Optional[List[str]] = None,
            env_vars: Optional[List[k8s.V1EnvVar]] = None,
            env_from_secret_ref: Optional[str] = None,
            labels: Optional[Dict] = None,
            startup_timeout_seconds: int = 120,
            resources: Optional[k8s.V1ResourceRequirements] = default_resources,
            termination_grace_period: Optional[int] = 60,
    ) -> None:
        env_from = None
        if env_from_secret_ref:
            env_from = [k8s.V1EnvFromSource(secret_ref=env_from_secret_ref)]
        super().__init__(
            labels=labels,
            namespace=namespace,
            image=image,
            name=name,
            cmds=cmds,
            arguments=arguments,
            env_vars=env_vars,
            env_from=env_from,
            in_cluster=True,
            startup_timeout_seconds=startup_timeout_seconds,
            hostnetwork=True,
            resources=resources,
            termination_grace_period=termination_grace_period,
            is_delete_operator_pod=True,
        )

    def set_env_vars_from_context(self, context):
        if self.env_vars is None:
            self.env_vars = []
        # TODO: set airflow_ts
        context.get()

    def execute(self, context) -> Optional[str]:
        self.set_env_vars_from_context(context)
        return super(BaseSparkKubernetesPodOperator, self).execute(context)
