from typing import List


class PolicyConfig:

    def __init__(self, role_name: str, file_name: str, git_module: str, tag: str, queue: str, rmm_permission: str, images: List[str]):
        self.role_name = role_name
        self.file_name = file_name
        self.git_module = git_module
        self.tag = tag
        self.queue = queue
        self.images = images
        self.rmm_permission = rmm_permission


class PolicyConfigFactory:

    def __init__(self, configs: List[PolicyConfig], protected_images: List[str]):
        self.configs = configs
        self.protected_images = protected_images
        self.validate()

        self.protected_queue_dict = {}
        for config in self.configs:
            file_name_set = self.protected_queue_dict.get(config.queue, set())
            file_name_set.add(config.file_name)
            self.protected_queue_dict[config.queue] = file_name_set

        self.protected_tag_dict = {}
        for config in self.configs:
            tag_set = self.protected_tag_dict.get(config.tag, set())
            tag_set.add(config.file_name)
            self.protected_tag_dict[config.tag] = tag_set

        self.protected_image_dict = {}
        for image in self.protected_images:
            self.protected_image_dict[image] = set()
        for config in self.configs:
            for image in config.images:
                if image in self.protected_image_dict:
                    self.protected_image_dict[image].add(config.file_name)

    def validate(self):
        role_name_set = set()
        file_name_set = set()
        git_module_set = set()
        tag_set = set()
        queue_set = set()
        rmm_permission_set = set()
        for config in self.configs:
            if config.role_name in role_name_set:
                raise Exception(f"duplicated role_name: {config.role_name}")
            role_name_set.add(config.role_name)
            if config.file_name in file_name_set:
                raise Exception(f"duplicated file_name: {config.file_name}")
            file_name_set.add(config.file_name)
            if config.git_module in git_module_set:
                raise Exception(f"duplicated git_module: {config.git_module}")
            git_module_set.add(config.git_module)
            if config.tag in tag_set:
                raise Exception(f"duplicated tag: {config.tag}")
            tag_set.add(config.tag)
            if config.queue in queue_set:
                raise Exception(f"duplicated queue: {config.queue}")
            queue_set.add(config.queue)
            if config.rmm_permission in rmm_permission_set:
                raise Exception(f"duplicated rmm_permission: {config.rmm_permission}")
            rmm_permission_set.add(config.rmm_permission)
            for image in config.images:
                if image not in self.protected_images:
                    raise Exception(f"image {image} is not protected")
