from typing import Callable, Dict, Set

from airflow.exceptions import AirflowClusterPolicyViolation
from airflow.models.dag import DAG


def dag_policy_builder(
        protected_tag_dict: Dict[str, Set[str]],
) -> Callable[[DAG], None]:
    def dag_policy(dag: DAG) -> None:
        file_path = dag.filepath
        tags = dag.tags
        if tags:
            for tag in tags:
                if tag in protected_tag_dict:
                    if file_path not in protected_tag_dict[tag]:
                        raise AirflowClusterPolicyViolation(f"dag: {dag} got invalid tag: {tag}")

    return dag_policy
