from typing import Callable, Dict, Set

from airflow.exceptions import AirflowClusterPolicyViolation
from airflow.models import BaseOperator


def task_policy_builder(
        protected_queue_dict: Dict[str, Set[str]],
        protected_image_dict: Dict[str, Set[str]],
) -> Callable[[BaseOperator], None]:
    def dag_policy(task: BaseOperator):
        file_path = task.dag.filepath
        queue = task.queue
        image = getattr(task, "image", None)
        if queue in protected_queue_dict:
            if file_path not in protected_queue_dict[queue]:
                raise AirflowClusterPolicyViolation(f"task: {task} got invalid queue: {queue}")
        if image:
            if image in protected_image_dict:
                if file_path not in protected_image_dict[image]:
                    raise AirflowClusterPolicyViolation(f"task: {task} got invalid image: {image}")

    return dag_policy
