from airflow.models.dagbag import DagBag
from typing import Dict, Optional


class Validator:

    def __init__(self, dag_folder: Optional[str]):
        self.dagbag = DagBag(
            dag_folder=dag_folder,
            include_examples=False,
            include_smart_sensor=False,
            safe_mode=False,
            read_dags_from_db=False,
            store_serialized_dags=False,
            load_op_links=False
        )

    @property
    def has_import_error(self) -> bool:
        return len(self.dagbag.import_errors) > 0

    @property
    def import_errors(self) -> Dict[str, str]:
        return self.dagbag.import_errors


if __name__ == '__main__':
    import sys
    validator = Validator(sys.argv[1] if len(sys.argv) > 1 else None)
    if validator.has_import_error:
        for file_path, msg in validator.import_errors.items():
            print(f"[ERROR] {file_path}: {msg}")
        sys.exit(1)
    else:
        print("[SUCCESS]")
